@@ -1,6 +1,6 @@
 ;;; Czech support for Festival
 
-;; Copyright (C) 2003, 2004, 2005, 2006 Brailcom, o.p.s.
+;; Copyright (C) 2003, 2004, 2005, 2006, 2007 Brailcom, o.p.s.
 
 ;; Author: Milan Zamazal <pdm@brailcom.org>
 
@@ -370,6 +370,7 @@
   ( Vowel [  ] Vowel = j i: j )
   ( [ IY ] Vowel = i j )
   ( [  ] Vowel = i: j )
+  ( IY [ IY ] = i )
   ( Vowel [ IY ] = j )
   ( Vowel [  ] = j i: )
   ;; Some vowel-vowel pairs
@@ -492,19 +493,11 @@
 
 ;;; Token to words processing
 
-(defvar czech-chars "a-zA-Zةݮ")
+(defvar czech-chars "a-zA-Z߻ةݮ")
 (defvar czech-char-regexp (string-append "[" czech-chars "]"))
 
 (defvar czech-multiword-abbrevs
-  '(("" ("dlouh" "a"))
-    ("" ("dlouh" "e"))
-    ("" ("dlouh" "i"))
-    ("" ("dlouh" "o"))
-    ("" ("dlouh" "u"))
-    ("" ("u" "s" "kroukem"))
-    ("w" ("dvojit" "v"))
-    ("" ("dlouh" "y"))
-    ("`" ("obrcen" "apostrof"))
+  '(("`" ("obrcen" "apostrof"))
     ("\\" ("zptn" "lomtko"))
     (">" ("vt" "ne"))
     ("<" ("men" "ne"))
@@ -740,6 +733,7 @@
          (czech-item.feat? token 'punc ".")
          (item.next token)
          (not (string-matches (item.feat token "n.whitespace") "  +")))
+    (item.set_feat token 'punc "")
     (if (not (czech-item.has-feat token 'punctype))
         (item.set_feat token 'punctype 'num))
     (append (czech-number* token name)
@@ -797,7 +791,7 @@
    ;; Abbreviations and other unpronouncable words
    ((and (string-matches
           name
-          "^[bcdfghjklmnpqrstvwxzBCDFGHJKLMNPQSTVWXZة][bcdfghjkmnpqstvwxzBCDFGHJKMNPQSTVWXZة]+$")
+          "^[bcdfghjklmnpqrstvwxzBCDFGHJKLMNPQSTVWXZ߻ة][bcdfghjkmnpqstvwxzBCDFGHJKMNPQSTVWXZ߻ة]+$")
          (not (lex.lookup_all name)))
     (mapcar (lambda (phoneme) `((name ,phoneme) (pos sym)))
             (lts.apply name 'czech-normalize)))
@@ -984,6 +978,9 @@
       (set! w (item.prev w)))
     result))
 
+(define (czech-pos-first-in-phrase? word)
+  (<= (czech-pos-in-phrase-from word) 1))
+
 (define (czech-pos-in-phrase-to word)
   (let ((result 1)
         (w word))
@@ -1023,6 +1020,9 @@
                    1)
              (item.set_feat w 'pos nil)
              (item.set_feat w 'pos 'punc)))
+        ;; Special interjections
+        ((member name '("" ""))
+         (item.set_feat w 'pos (if (czech-pos-first-in-phrase? w) 'int 'sym)))
         ;; Single letter, not in the role of a word
         ((and (eq? (string-length name) 1)
               (czech-pos-last-in-phrase? w))
@@ -1358,9 +1358,9 @@
 (define (czech-identify-stress-units sylwords)
   (let ((units (mapcar list sylwords))
         (unit-word (lambda (unit)
-                     (and (eqv? (length unit) 1)
+                     (and (not (null? unit))
                           (item.parent
-                           (item.relation (car unit) 'SylStructure)))))
+                           (item.relation (car (last unit)) 'SylStructure)))))
         (unit-word-name (lambda (unit)
                           (and (eqv? (length unit) 1)
                                (item.feat (car unit)
@@ -1374,16 +1374,16 @@
         (begin
           ;; Basic joining    
           (let ((units* units))
-            (while units*
+            (while (cdr units*)
               (let ((w (unit-word (car units*))))
                 (if (or ;; Join non-syllabic prepositions
-                     (czech-item.feat? w 'pos 'prep0)
-                     ;; Join proper single-syllabic prepositions
-                     (and (member (czech-downcase (item.name w))
-                                  czech-proper-single-syl-prepositions)
-                          (not (czech-item.feat? w "pos" "se"))))
-                    (merge units*)))
-              (set! units* (cdr units*))))
+                        (czech-item.feat? w 'pos 'prep0)
+                        ;; Join proper single-syllabic prepositions
+                        (and (member (czech-downcase (item.name w))
+                                     czech-proper-single-syl-prepositions)
+                             (not (czech-item.feat? w "pos" "se"))))
+                    (merge units*)
+                    (set! units* (cdr units*))))))
           ;; At most 1 word now?
           (if (<= (length units) 1)
               units
@@ -1688,7 +1688,13 @@
           (let ((nsyls (czech-unit-syllable-count unit)))
             (let ((contour (czech-random-choice
                             (cdr (assoc (list contourtype
-                                              (if (<= nsyls 6) nsyls 6))
+                                              (cond
+                                               ((< nsyls 1)
+                                                1)
+                                               ((> nsyls 6)
+                                                6)
+                                               (t
+                                                nsyls)))
                                         czech-int-contours)))))
               ;; Adjust the first syllables of final contours
               (if (or (string-equal position "F")
